@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 import argparse
 import shlex
 import subprocess
@@ -12,6 +11,7 @@ from pathlib import Path
 from tempfile import NamedTemporaryFile
 import abc
 from functools import wraps
+import sys
 
 
 here = Path(__file__).parent
@@ -43,7 +43,7 @@ def lazy(f):
 class Sudo:
     def __init__(self):
         self.bg = subprocess.Popen(
-            ["sudo", "python", here / "sudobg.py"],
+            ["sudo", sys.executable, here / "sudobg.py"],
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
         )
@@ -57,7 +57,6 @@ class Sudo:
             return str(o)
         return o
 
-    @decolog
     def _req(self, *pargs, **kwargs):
         self.bg.stdin.write(
             (json.dumps(self._encode([pargs, kwargs])) + "\n").encode("utf-8")
@@ -142,7 +141,11 @@ class SnapPlugin(Plugin):
         return "snap"
 
     def installed(self):
-        return check_output(["snap", "list"])
+        try:
+            return check_output(["snap", "list"])
+        except:
+            logging.warning("Failed to get snap packages", exc_info=True)
+            return []
 
     def install(self, names):
         for name in names:
@@ -166,7 +169,6 @@ def main():
     parser = argparse.ArgumentParser(
         description="Debianish Linux declarative package management"
     )
-    parser.add_argument("conf", help="Configuration path", default="decapt.luxem")
 
     subparsers = parser.add_subparsers(title="Command", dest="_command")
 
@@ -177,6 +179,7 @@ def main():
     com_gen.add_argument(
         "-f", "--force", help="Overwrite existing configuration", action="store_true",
     )
+    com_gen.add_argument("conf", help="Configuration path", default="decapt.luxem")
 
     com_sync = subparsers.add_parser(
         "sync", description="Install, remove, and upgrade packages. Default command.",
@@ -187,6 +190,7 @@ def main():
         help="Show changes to be implemented on next sync",
         action="store_true",
     )
+    com_sync.add_argument("conf", help="Configuration path", default="decapt.luxem")
 
     args = parser.parse_args()
 
@@ -213,7 +217,7 @@ def main():
     elif command == "sync":
         with open(args.conf, "r") as conff:
             conf = luxem.load(conff)[0]
-        installed = dict()
+        want = dict()
         add = dict()
         remove = dict()
         for package in conf["installed"]:
@@ -224,22 +228,34 @@ def main():
                 plugin = DEFAULT_PLUGIN
             if isinstance(package, dict):
                 package = package["name"]
-            installed.setdefault(plugin, set()).add(package)
+            want.setdefault(plugin, set()).add(package)
         for plugin in plugins:
             print(f"Scanning current package state for {plugin.id()}...")
-            want = set(plugin.installed())
-            add[plugin.id()] = want - installed.get(plugin.id(), set())
-            remove[plugin.id()] = installed.get(plugin.id(), set()) - want
+            installed = set(plugin.installed())
+            add[plugin.id()] = want.get(plugin.id(), set()) - installed
+            remove[plugin.id()] = installed - want.get(plugin.id(), set())
 
         if args.dry_run:
-            print("\nPlan: Adding--\n{}".format(luxem.dumps(add)))
-            print("\nPlan: Removing--\n{}".format(luxem.dumps(remove)))
+            print(
+                "\nPlan: Adding--\n{}".format(
+                    luxem.dumps(add, pretty=True).decode("utf-8")
+                )
+            )
+            print(
+                "\nPlan: Removing--\n{}".format(
+                    luxem.dumps(remove, pretty=True).decode("utf-8")
+                )
+            )
         else:
             for plugin in plugins:
                 print(f"Installing packages for {plugin.id()}...")
-                plugin.install(add.get(plugin.id(), set()))
+                plugin_add = list(add.get(plugin.id(), set()))
+                if plugin_add:
+                    plugin.install(plugin_add)
                 print(f"Removing packages for {plugin.id()}...")
-                plugin.remove(remove.get(plugin.id(), set()))
+                plugin_remove = list(remove.get(plugin.id(), set()))
+                if plugin_remove:
+                    plugin.uninstall(plugin_remove)
 
             if not add:
                 print("No packages to install.")
